/*
 * MAX2SQ : Max 2.0 AREAS.DAT to SQUISH.CFG or SQAFIX.CFG convertor
 *
 * Purpose: This is quick and dirty utility intended to help those
 *          lazy ones who hate editing, to convert Max 2.0 message
 *          area definitions to a Squish or SqaFix format. Just
 *          redirect output of this program to a temporary file,
 *          and insert the result into the appropriate *.CFG file
 *
 * Created: 12/Jan/92
 * Updated: 12/Jan/92
 *
 * Written by Pete Kvitek of JV Dialogue 1st BBS (2:5020/6) +7-095-329-2192
 * Copyright (c) 1991 by JV DIALOGUE. All rights reserved.
 *
 */

 // Pick up the standard stuff

 #include <dir.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>

 // Here are the Maximus and Squish headres. Make sure you have
 // unpacked the MSGAPI0.* and \MSGAPI\INCLUDE is somewere in
 // include files path for the compiler...

 #include "mstruct.h"
 #define __TYPEDEFS_H_DEFINED		// !! Scott missed this... !!
 #define __STAMP_H_DEFINED
 #include "msgapi.h"

/*
 * Module declarations
 */

 // Miscellaneous defines

 #define VERSION       "1.01"		// Software release version
 #define MD_SQUISH	1		// Produce Squish style area
 #define MD_SQAFIX	2		// Produce SqaFix style area

 // Module variables

 static char achPriv[] = "DLNWPFECASH";		// Privilage/Area group table
 static char achAreaFile[MAXPATH] = "AREA.DAT";	// Maximus AREA.DAT file name
 static int  fMode;				// Operation mode (MD_)

/*
 * This subroutine displays a logo screen
 */

 static void DoShowLogo(void)
 {
   fprintf(stderr,
   "\n"
   "Maximus to Squish/SqaFix Area Definition Convertor v"VERSION"\n"
   "Written by Pete Kvitek of JV Dialogue 1st BBS, 2:5020/6\n"
   "Copyright (C) 1991 by JV Dialogue. All rights reserved.\n"
   "\n"
   );
 }

/*
 * This subroutine displays a help screen
 */

 static void DoShowHelp(void)
 {
   fprintf(stderr,
   "Usage:    MAX2SQ {Squish|SqaFix} [AREA.DAT]\n"
   "\n"
   "          Redirect the output to the temporary file\n"
   "          and use it as a template for your Squish or\n"
   "          SqaFix echo area definitions\n"
   "\n"
   "Examples: Max2Sq Squish \\max\\area.dat > sqsh.cfg\n"
   "          or\n"
   "          Max2Sq SqaFix \\max\\area.dat > sqaf.cfg\n"
   );
 }

/*
 * This subroutine processes the command line parameters
 */

 static void DoProcCmdLine(short cArg, char * apszArg[])
 {
   // Display the logo screen

   DoShowLogo();

   // Check if there are command line parameters and if no,
   // display the help screen and terminate right away

   if (cArg < 2) {
     DoShowHelp();
     exit(EXIT_SUCCESS);
   }

   // Check what operation mode is requested if any

   if (cArg >= 2) {
     if (!stricmp(apszArg[1], "Squish"))
       fMode = MD_SQUISH;
     else
     if (!stricmp(apszArg[1], "SqaFix"))
       fMode = MD_SQAFIX;
     else {
       fprintf(stderr, "Invalid operation mode '%s'\n", apszArg[1]);
     }
   }

   // Get the AREA.DAT file name override if any

   if (cArg >= 3) {
     strncpy(achAreaFile, apszArg[2], sizeof(achAreaFile) - 1);
     strupr(achAreaFile);
   }
 }

/*
 * Main...
 */

 void main(int argc, char *argv[])
 {
   struct _area area;
   short iArea, cbArea = 0;
   FILE * pfile;

   // Scan in the command line

   DoProcCmdLine(argc, argv);

   // Open the areas definition file and check if ok

   if ((pfile = fopen(achAreaFile, "rb")) == NULL) {
     fprintf(stderr, "Can't open: %s\n", achAreaFile);
     exit(EXIT_FAILURE);
   }

   // Print out header according to the mode we're running in

   if (fMode == MD_SQAFIX)
     printf(";        AreaTag\t\tGroup\tDescription\n"
	    ";        -------\t\t-----\t-----------\n");
   else
   if (fMode == MD_SQUISH)
     printf(";        AreaTag\t\tPath\t\tFlags\n"
	    ";        -------\t\t----\t\t-----\n");

   // Loop through all the area definitions

   for (iArea = 0; !fseek(pfile, iArea * (long)cbArea, SEEK_SET); iArea++) {

     // Read in the subsequent area definition and check if ok

     if (fread(&area, sizeof(struct _area), 1, pfile) != 1)
       if (feof(pfile))
	 break;
       else {
	 fprintf(stderr, "Can't read: %s\n", achAreaFile);
	 exit(EXIT_FAILURE);
       }

     // Store the size of the area definition ob the first turn

     if (iArea == 0)
       cbArea = area.struct_len;

     // Process all the echo areas using the requested mode method
     // Note: the SqaFix area group is derived from the privilage
     // level required to access the particular area

     if (area.attrib[0] & ECHO) {

       printf("EchoArea %-15s\t", area.msgname);

       if (fMode == MD_SQAFIX) {
	 printf("%c\t\"%s\"\n",
		 achPriv[min(area.msgpriv, sizeof(achPriv) - 1)], // Group
		 area.msginfo);					  // Descr
       } else

       if (fMode == MD_SQUISH) {
	 printf("%s\t", area.msgpath);				  // Path
	 if (area.type == MSGTYPE_SQUISH) {                       // Flags
	   printf("-$");
	   if (area.killbyage > 0) printf("d%u", area.killbyage);
	   if (area.killbynum > 0) printf("m%u", area.killbynum);
	 }
	 printf("\n");
       }
     }
   }

   // Close the areas defintion file and exit... Woops, this should
   // work right away! About ten minutes to type this code in is mucha
   // better then editing of my 50+ echo mail area definitions :-)
   // Ohhh... the LICENCE: hereby I claim this code PUBLIC DOMAIN!
   // And, THANX SCOTT FOR GIVING THE EASY ACCESS TO THE MAX'S GUTS!

   fclose(pfile);
   fprintf(stderr, "\nDone...\n");

   exit(EXIT_SUCCESS);
 }

/*
 * End of MAX2SQ.C
 */